--#############################################################################
--
-- Headline: Simulation with test bench in single file


--#############################################################################
-- Entity

library ieee;
use ieee.std_logic_1164.all;

entity mdl_tb is
end entity;


--#############################################################################
-- Architecture

library ieee;
use ieee.numeric_std.all;

architecture sim of mdl_tb is


  --===========================================================================
  -- Clock and reset decl.

  -- Clock
  constant CLK_FREQ : real := 100.0E6;  -- Clock frequency in Hz
  signal clk        : std_logic;

  -- Reset
  constant RST_PER : time := 100 ns;  -- Reset period; and then waiting for rising clk edge before deassert rst
  signal rst       : std_logic;


  --===========================================================================
  -- Counter impl.

  signal cnt : std_logic_vector(3 downto 0);


  --===========================================================================
  -- DUT decl.

  signal dut_a      : std_logic;
  signal dut_z_proc : std_logic;
  signal dut_z_conc : std_logic;


  --===========================================================================
  -- Test control decl.

  -- None


begin


  --===========================================================================
  -- Clock and reset impl.

  -- Clock generation
  process is
  begin
    while TRUE loop
      clk <= '1';
      wait for 0.5 sec / CLK_FREQ;
      clk <= '0';
      wait for (1.0 sec / CLK_FREQ) - (0.5 sec / CLK_FREQ);
    end loop;
  end process;

  -- Reset generation
  process is
  begin
    rst <= '1';
    wait for RST_PER;
    wait until rising_edge(clk);
    rst <= '0';
    wait;
  end process;


  --===========================================================================
  -- Counter impl.

  -- Counter
  process (clk) is
  begin
    if rising_edge(clk) then
      cnt <= std_logic_vector(unsigned(cnt) + 1);
    end if;
    if rst = '1' then
      cnt <= (others => '0');
    end if;
  end process;

  -- Drive dut_a
  dut_a <= cnt(2);


  --===========================================================================
  -- DUT impl.

  mdl_e : entity work.mdl
    port map(
      clk_i => clk,
      a_i => dut_a,
      z_proc_o => dut_z_proc,
      z_conc_o => dut_z_conc);


  --===========================================================================
  -- Test control impl.

  process is
  begin
    -- Run for a while
    wait for 1 us;
    -- End of simulation
    report "OK   ########## Sim end: OK :-) ##########   (not actual failure)" severity FAILURE;
    wait;
  end process;

end architecture;


--#############################################################################
-- EOF
